@@ -9,20 +9,17 @@
 
     Powered by the awesome Babel and pytz libraries for the l10n part.
 
-
 """
 from __future__ import absolute_import
 import os
-
 # Workaround for a OSX bug
 if os.environ.get('LC_CTYPE', '').lower() == 'utf-8':
     os.environ['LC_CTYPE'] = 'en_US.utf-8'
-import datetime as d
+
+import datetime as dt
 from decimal import Decimal
-import io
-from os.path import (join, dirname, realpath, abspath, normpath, isdir, isfile,
-    splitext)
-import re
+from os.path import (join, dirname, realpath, abspath, normpath, isdir,
+                     isfile, splitext)
 
 from babel import dates, numbers, Locale
 try:
@@ -31,29 +28,31 @@ except ImportError:
     try:
         from jinja2 import Markup
     except ImportError:
-        Markup = lambda x: x
+        try:
+            from mako.filters import markupsafe
+            Markup = markupsafe.Markup
+        except ImportError:
+            Markup = lambda x: x
 from pytz import timezone, UTC
-import yaml
+
+from .reader import get_data
+from .utils import (
+    normalize_locale,
+    get_werkzeug_preferred_languages,
+    get_webob_preferred_languages,
+    get_django_preferred_languages,
+    get_werkzeug_ua_language,
+    get_django_ua_language)
 
 
-
-__version__ = '0.3'
+__version__ = '0.5.1'
 
 
 LOCALES_DIR = 'locales'
 
-class InvalidLocale(Exception):
-    pass
-
-
-# For parsing raw 'User-Agent' headers
-_language_re = re.compile(
-    r'(?:;\s*|\s+)(\b\w{2}\b(?:-\b\w{2}\b)?)\s*;|'
-    r'(?:\(|\[|;)\s*(\b\w{2}\b(?:-\b\w{2}\b)?)\s*(?:\]|\)|;)'
-)
-
 
 class I18n(object):
+
     """Main class
 
     """
@@ -75,11 +74,10 @@ class I18n(object):
         'datetime.full': None,
         'datetime.long': None,
     }
-    
 
     def __init__(self, locales_dirs=None, get_request=None,
-            default_locale=None, default_timezone=None, date_formats=None,
-            markup=Markup):
+                 default_locale=None, default_timezone=None, date_formats=None,
+                 markup=Markup):
         """
         locales_dirs
         :   list of paths that will be searched, in order,
@@ -94,7 +92,7 @@ class I18n(object):
         :   overwrite the defaults date formats.
         markup
         :   overwrite the function used by `translate` to flags HTML code
-            as 'safe'. `jinja2.Markup` is used by default.
+            as 'safe'. `markupsafe.Markup` is used by default.
 
         """
         self.get_request = get_request
@@ -117,7 +115,6 @@ class I18n(object):
         self.list_available_languages()
         self.translations = {}
 
-
     def set_defaults(self, default_locale, default_timezone):
         """Set the default locale from the configuration as an instance of
         `Babel.Locale` and the default timezone as a `pytz.timezone` object.
@@ -127,7 +124,6 @@ class I18n(object):
         self.default_locale = default_locale
         self.default_timezone = timezone(default_timezone or 'utc')
 
-
     def list_available_languages(self):
         """Make a list of all available languages in the locales dir to use it
         later for content negotiation.
@@ -144,7 +140,6 @@ class I18n(object):
 
         self.available_languages = languages
 
-
     def get_request_timezone(self, request, default=UTC):
         """Returns the timezone that should be used for this request as a
         `DstTzInfo` instance.
@@ -156,16 +151,16 @@ class I18n(object):
 
         """
         tzinfo = (
-                getattr(request, 'tzinfo', None) or
-                getattr(request, 'args', getattr(request, 'GET', {})).get('tzinfo')
-            ) or default
+            getattr(request, 'tzinfo', None) or
+            getattr(request, 'args', getattr(
+                    request, 'GET', {})).get('tzinfo')
+        ) or default
 
         if isinstance(tzinfo, basestring):
             tzinfo = timezone(tzinfo)
         request.tzinfo = tzinfo
         return request.tzinfo
 
-
     def get_request_locale(self, request, default='en'):
         """Returns the locale that should be used for this request as a
         `babel.Locale` instance.
@@ -187,15 +182,18 @@ class I18n(object):
                 get_webob_preferred_languages(request) or
                 get_django_preferred_languages(request)
             )
-            # To ensure a consistent matching, Babel algorithm is used.
-            locale = Locale.negotiate(preferred, self.available_languages, sep='-')
+            if preferred:
+                # To ensure a consistent matching, Babel algorithm is used.
+                locale = Locale.negotiate(preferred,
+                                          self.available_languages,
+                                          sep='-')
 
         if not locale:
             locale = (
                 get_werkzeug_ua_language(request) or
                 get_django_ua_language(request)
             )
-        
+
         locale = locale or default
 
         if isinstance(locale, basestring):
@@ -204,7 +202,6 @@ class I18n(object):
         request.locale = locale
         return request.locale
 
-
     def get_locale(self):
         """Returns the locale that should be used for this request as
         an instance of `Babel.Locale`.
@@ -216,7 +213,6 @@ class I18n(object):
             return self.default_locale
         return self.get_request_locale(request, self.default_locale)
 
-
     def get_timezone(self):
         """Returns the timezone that should be used for this request as
         `pytz.timezone` object.  This returns the default timezone if used
@@ -228,7 +224,6 @@ class I18n(object):
             return self.default_timezone
         return self.get_request_timezone(request, self.default_timezone)
 
-
     def load_language(self, path, locale):
         """From the given `path`, load the language file for the current or
         given locale.  If the locale has a territory attribute (eg: 'US') the
@@ -252,13 +247,11 @@ class I18n(object):
         else:
             return
         try:
-            with io.open(filename) as f:
-                data = yaml.load(f)
-            self.translations[cache_key] = data
-            return data
+            data = get_data(filename)
         except (IOError, AttributeError):
             return
-
+        self.translations[cache_key] = data
+        return data
 
     def find_keypath(self, key):
         """Based on the `key`, teturn the path of the language file and the
@@ -283,7 +276,6 @@ class I18n(object):
             return None, None
         return path, subkey
 
-
     def key_lookup(self, key, locale):
         """
         """
@@ -304,7 +296,6 @@ class I18n(object):
         except (IndexError, ValueError):
             return None
 
-
     def translate(self, key, count=None, locale=None, **kwargs):
         """Load the translation for the given key using the current locale.
 
@@ -334,7 +325,7 @@ class I18n(object):
         value = self.key_lookup(key, locale)
         if not value:
             return self.markup('<missing:%s>' % (key, ))
-        
+
         if isinstance(value, dict):
             value = self.pluralize(value, count)
 
@@ -346,7 +337,6 @@ class I18n(object):
 
         return value
 
-
     @property
     def lazy_translate(self):
 
@@ -361,27 +351,26 @@ class I18n(object):
 
         return LazyWrapper
 
-
-    def pluralize(self, d, count):
+    def pluralize(self, dic, count):
         """Takes a dictionary and a number and return the value whose key in
         the dictionary is that number.  If that key doesn't exist, a `'n'` key
         is tried instead.  If that doesn't exits either, an empty string is
         returned.  Examples:
 
             >>> i18n = I18n()
-            >>> d = {
+            >>> dic = {
                 0: u'No apples',
                 1: u'One apple',
                 3: u'Few apples',
                 'n': u'%(count)s apples',
                 }
-            >>> i18n.pluralize(d, 0)
+            >>> i18n.pluralize(dic, 0)
             'No apples'
-            >>> i18n.pluralize(d, 1)
+            >>> i18n.pluralize(dic, 1)
             'One apple'
-            >>> i18n.pluralize(d, 3)
+            >>> i18n.pluralize(dic, 3)
             'Few apples'
-            >>> i18n.pluralize(d, 10)
+            >>> i18n.pluralize(dic, 10)
             '%(count)s apples'
             >>> i18n.pluralize({0: 'off', 'n': 'on'}, 3)
             'on'
@@ -394,8 +383,7 @@ class I18n(object):
         if count is None:
             count = 0
         scount = str(count)
-        return d.get(count, d.get(scount, d.get('n', u'')))
-    
+        return dic.get(count, dic.get(scount, dic.get('n', u'')))
 
     def to_user_timezone(self, datetime, tzinfo=None):
         """Convert a datetime object to the user's timezone.  This
@@ -410,7 +398,6 @@ class I18n(object):
         tzinfo = tzinfo or self.get_timezone()
         return tzinfo.normalize(datetime.astimezone(tzinfo))
 
-
     def to_utc(self, datetime, tzinfo=None):
         """Convert a datetime object to UTC and drop tzinfo.  This is the
         opposite operation to `to_user_timezone`.
@@ -421,7 +408,6 @@ class I18n(object):
             datetime = tzinfo.localize(datetime)
         return datetime.astimezone(UTC).replace(tzinfo=None)
 
-
     def _get_format(self, key, format):
         """A small helper for the datetime formatting functions.  Looks up
         format defaults for different kinds.
@@ -435,9 +421,8 @@ class I18n(object):
                 format = rv
         return format
 
-
     def _date_format(self, formatter, obj, format, rebase,
-            locale=None, tzinfo=None, **extra):
+                     locale=None, tzinfo=None, **extra):
         """Internal helper that formats the date.
 
         """
@@ -447,40 +432,38 @@ class I18n(object):
             extra['tzinfo'] = tzinfo or self.get_timezone()
         return formatter(obj, format, locale=locale, **extra)
 
-
     def format(self, value, *args, **kwargs):
         """Return a formatted `value` according to the detected type and
         given parameters.  It doesn't know anything about currency, percent or
         scientific formats, so use the other methods for those cases.
-        
+
         """
         locale = kwargs.pop('locale', None)
         tzinfo = kwargs.pop('tzinfo', None)
 
-        if isinstance(value, d.date):
-            if isinstance(value, d.datetime):
+        if isinstance(value, dt.date):
+            if isinstance(value, dt.datetime):
                 return self.format_datetime(value, locale=locale,
-                    tzinfo=tzinfo, *args, **kwargs)
+                                            tzinfo=tzinfo, *args, **kwargs)
             else:
-                return self.format_date(value, locale=locale, tzinfo=tzinfo, 
-                    *args, **kwargs)
+                return self.format_date(value, locale=locale, tzinfo=tzinfo,
+                                        *args, **kwargs)
 
         if isinstance(value, int):
             return self.format_number(value, locale=locale, *args, **kwargs)
         if isinstance(value, (float, Decimal)):
             return self.format_decimal(value, locale=locale, *args, **kwargs)
 
-        if isinstance(value, d.time):
+        if isinstance(value, dt.time):
             return self.format_time(value, locale=locale, tzinfo=tzinfo,
-                *args, **kwargs)
-        if isinstance(value, d.timedelta):
+                                    *args, **kwargs)
+        if isinstance(value, dt.timedelta):
             return self.format_timedelta(value, locale=locale, *args, **kwargs)
 
         return value
 
-
     def format_datetime(self, datetime=None, format=None, rebase=True,
-            locale=None, tzinfo=None):
+                        locale=None, tzinfo=None):
         """Return a date formatted according to the given pattern.  If no
         `datetime.datetime` object is passed, the current time is
         assumed.  By default rebasing happens which causes the object to
@@ -499,11 +482,10 @@ class I18n(object):
         """
         format = self._get_format('datetime', format)
         return self._date_format(dates.format_datetime, datetime, format,
-            rebase, locale=locale, tzinfo=tzinfo)
-
+                                 rebase, locale=locale, tzinfo=tzinfo)
 
     def format_date(self, date=None, format=None, rebase=True,
-            locale=None, tzinfo=None):
+                    locale=None, tzinfo=None):
         """Return a date formatted according to the given pattern.  If no
         `datetime.datetime` or `datetime.date` object is passed,
         the current time is assumed.  By default rebasing happens which causes
@@ -520,15 +502,14 @@ class I18n(object):
         named `dateformat`.
 
         """
-        if rebase and isinstance(date, d.datetime):
+        if rebase and isinstance(date, dt.datetime):
             date = self.to_user_timezone(date, tzinfo=tzinfo)
         format = self._get_format('date', format)
         return self._date_format(dates.format_date, date, format, rebase,
-            locale=locale, tzinfo=tzinfo)
-
+                                 locale=locale, tzinfo=tzinfo)
 
     def format_time(self, time=None, format=None, rebase=True,
-            locale=None, tzinfo=None):
+                    locale=None, tzinfo=None):
         """Return a time formatted according to the given pattern.  If no
         `datetime.datetime` object is passed, the current time is
         assumed.  By default rebasing happens which causes the object to
@@ -547,11 +528,10 @@ class I18n(object):
         """
         format = self._get_format('time', format)
         return self._date_format(dates.format_time, time, format, rebase,
-            locale=locale, tzinfo=tzinfo)
-
+                                 locale=locale, tzinfo=tzinfo)
 
     def format_timedelta(self, datetime_or_timedelta, granularity='second',
-            locale=None):
+                         locale=None):
         """Format the elapsed time from the given date to now or the given
         timedelta.
 
@@ -560,30 +540,29 @@ class I18n(object):
 
         """
         locale = normalize_locale(locale) or self.get_locale()
-        if isinstance(datetime_or_timedelta, d.datetime):
-            datetime_or_timedelta = d.datetime.utcnow() - datetime_or_timedelta
+        if isinstance(datetime_or_timedelta, dt.datetime):
+            datetime_or_timedelta = dt.datetime.utcnow(
+            ) - datetime_or_timedelta
         return dates.format_timedelta(datetime_or_timedelta, granularity,
-            locale=locale)
-
+                                      locale=locale)
 
     def format_number(self, number, locale=None):
         """Return the given number formatted for the locale in the
         current request.
-        
+
         number
         :   the number to format
-        
+
         return (unicode)
         :   the formatted number
 
         This function is also available in the template context as filter
         named `numberformat`.
-        
+
         """
         locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_number(number, locale=locale)
 
-
     def format_decimal(self, number, format=None, locale=None):
         """Return the given decimal number formatted for the locale in the
         current request.
@@ -592,18 +571,17 @@ class I18n(object):
         :   the number to format
         format
         :   the format to use
-        
+
         return (unicode)
         :   the formatted number
 
         This function is also available in the template context as filter
         named `decimalformat`.
-        
+
         """
         locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_decimal(number, format=format, locale=locale)
 
-
     def format_currency(self, number, currency, format=None, locale=None):
         """Return the given number formatted for the locale in the
         current request.
@@ -614,20 +592,19 @@ class I18n(object):
         :   the currency code
         format
         :   the format to use
-        
+
         return (unicode)
         :   the formatted number
 
         This function is also available in the template context as filter
         named `currencyformat`.
-        
+
         """
         locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_currency(
             number, currency, format=format, locale=locale
         )
 
-
     def format_percent(self, number, format=None, locale=None):
         """Return a percent value formatted for the locale in the
         current request.
@@ -636,10 +613,10 @@ class I18n(object):
         :   the number to format
         format
         :   the format to use
-        
+
         return (unicode)
         :   the formatted percent number
-        
+
         This function is also available in the template context as filter
         named `percentformat`.
 
@@ -647,7 +624,6 @@ class I18n(object):
         locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_percent(number, format=format, locale=locale)
 
-
     def format_scientific(self, number, format=None, locale=None):
         """Return value formatted in scientific notation for the locale in
         the current request.
@@ -656,85 +632,13 @@ class I18n(object):
         :   the number to format
         format
         :   the format to use
-        
+
         return (unicode)
         :   the formatted percent number
-        
+
         This function is also available in the template context as filter
         named `scientificformat`.
 
         """
         locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_scientific(number, format=format, locale=locale)
-
-
-def normalize_locale(locale):
-    if isinstance(locale, Locale):
-        return locale
-    if isinstance(locale, basestring):
-        locale = locale.replace('_', '-')
-        return Locale.parse(locale, sep='-')
-    if isinstance(locale, tuple):
-        return Locale(*locale)
-    return
-        
-
-def get_werkzeug_preferred_languages(request):
-    """Return a list of preferred languages from a `werkzeug.wrappers.Request`
-    instance.
-
-    """
-    languages = getattr(request, 'accept_languages', None)
-    return languages.values() if languages else None
-
-
-def get_webob_preferred_languages(request):
-    """Return a list of preferred languages from a `webob.Request` instance.
-
-    """
-    languages = getattr(request, 'accept_language', None)
-    return list(languages) if languages else None
-    
-
-def get_django_preferred_languages(request):
-    """Take a `django.HttpRequest` instance and return a list of preferred
-    languages from the headers.
-
-    """
-    meta = getattr(request, 'META', None)
-    if not meta:
-        return None
-    header = request.META.get('HTTP_ACCEPT_LANGUAGE')
-    if not header:
-        return None
-    languages = [l.split(';')[0].strip() for l in header.split(',')]
-
-
-def get_werkzeug_ua_language(request):
-    """Return the language from the Werkzeug/Webob processed 'User-Agent'
-    header.
-
-    """
-    ua = getattr(request, 'user_agent', None)
-    if not ua:
-        return None
-    return getattr(request.user_agent, 'language', None)
-                
-
-def get_django_ua_language(request):
-    """Take a `django.HttpRequest` instance and try to extract the language
-    from the 'User-Agent' header.
-
-    """
-    meta = getattr(request, 'META', None)
-    if not meta:
-        return None
-    header = request.META.get('HTTP_USER_AGENT')
-    if not header:
-        return None
-    match = _language_re.search(header)
-    if match is not None:
-        return match.group(1) or match.group(2)
-    return None
-
-
